<?xml version="1.0"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text" indent="yes" />
  <!--  props.xsl
        Rick Landau, Dell, 20070212
        From Projector&Display Mgt WG XML for mgt group properties, 
        extract a fragment of SNMP MIB syntax. 
        
        In this case, extract the OBJECT-TYPE definition for each property.  
        -->
  <xsl:variable name="SNMPprefix">pdm</xsl:variable>
  <xsl:variable name="SNMPPrefix">Pdm</xsl:variable>
  <xsl:variable name="MIBname">pdmMIBObjects</xsl:variable>

<!-- G r o u p -->
  <xsl:template match="Group">
<!--    - seen:Group <xsl:value-of select="@GroupName"/>\n-->
    -- Processed by props25.xsl at   .  .  .  <!-- <xsl:value-of select="xsl:current-dateTime()"/> 
-->
<xsl:value-of select="$SNMPprefix"/><xsl:value-of select="@GroupName"/> OBJECT IDENTIFIER ::= { <xsl:value-of select="$MIBname"/> inserthere-index }
    <xsl:variable name="scalaroffsetfortableis">
      <xsl:choose>
        <xsl:when test="Table">1</xsl:when>
        <xsl:otherwise></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:apply-templates select="Property">
      <xsl:with-param name="GroupName"><xsl:value-of select="@GroupName"/></xsl:with-param>
      <xsl:with-param name="scalaroffsetfortableis"><xsl:value-of select="$scalaroffsetfortableis"/></xsl:with-param>
    </xsl:apply-templates>

    <xsl:apply-templates select="Table">
      <xsl:with-param name="GroupName"><xsl:value-of select="@GroupName"/></xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

<!-- T a b l e -->
  <xsl:template name="Table" match="Table">
    <xsl:param name="GroupName"/>
<!--    - seen:Table <xsl:value-of select="@TableName"/>\n-->
    <xsl:text>
</xsl:text>
    -- table at { <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName"/><xsl:text> </xsl:text><xsl:value-of select="position()"/> }
<!--
 --><xsl:variable name="myentryis">
      <xsl:value-of select="$SNMPprefix"/>
      <xsl:value-of select="@TableName"/>
      <xsl:if test="not(contains(@TableName,'Entry'))">Entry</xsl:if>
    </xsl:variable>
<!-- Index property that goes at the beginning of every table -->
<xsl:value-of select="$SNMPprefix"/>
    <xsl:value-of select="$GroupName"/>Index OBJECT-TYPE
    SYNTAX     Integer32 (1..2147483647)  -- Automatic index variable
    MAX-ACCESS not-accessible
    STATUS     current
    DESCRIPTION
      "A unique value used by the device to identify this
        entry.  Values are expected to remain
        stable across successive power cycles."
    ::= { <xsl:value-of select="$myentryis"/> 1 }
<!-- Find all Properties, inside or outside tables. -->
    <xsl:apply-templates select="Property">
      <xsl:with-param name="GroupName"><xsl:value-of select="$GroupName"/></xsl:with-param>
      <xsl:with-param name="TableName">
        <xsl:value-of select="@TableName"/>
        <xsl:if test="not(contains(@TableName,'Table'))">Table</xsl:if>
      </xsl:with-param>
      <xsl:with-param name="EntryName">
        <xsl:value-of select="@TableName"/>
        <xsl:if test="not(contains(@TableName,'Entry'))">Entry</xsl:if>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

<!-- P r o p e r t y -->
  <xsl:template name="Property" match="Property">
    <xsl:param name="GroupName"/>
    <xsl:param name="TableName"/>
    <xsl:param name="EntryName"/>
    <xsl:param name="scalaroffsetfortableis"/>
<!--    - Property at-AbstractName=<xsl:value-of select="@AbstractName"/>
    - parent is type=<xsl:value-of select="name(..)"/>
    - Property in table at-TableName=<xsl:value-of select="../@TableName"/>
    - Property in group at-name=<xsl:value-of select="../../@name"/>\n-->
    <!-- The vanilla version for property names. -->
    <xsl:variable name="mynameis">
      <xsl:choose>
        <xsl:when test="contains(SNMPName,' ')">
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="$GroupName"/>
          <xsl:value-of select="@AbstractName" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="SNMPName" />
        </xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    <!-- The capitalized version for use in TC names -->
    <xsl:variable name="myNameis">
      <xsl:choose>
        <xsl:when test="contains(SNMPName,' ')">
          <xsl:value-of select="$SNMPPrefix"/>
          <xsl:value-of select="$GroupName"/>
          <xsl:value-of select="@AbstractName" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$SNMPPrefix"/>
          <xsl:value-of select="SNMPName" />
        </xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>

<!--  G r o u p  o r  T a b l e  -->
<!-- What is the right level of context for this property? -->
    <!-- If this is a table column, position it within the table.
    Otherwise, position it within the containing group. -->
    <xsl:variable name="mycontextis">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName" />
        </xsl:when>
        <xsl:when test="name(..)='Table'">
          <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$EntryName" />
        </xsl:when>
        <xsl:otherwise>ErrorUnknownContext</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>

<!--  O I D   n u m b e r i n g  -->
<!-- What is the right numbering at this level in the OID for this property? -->
    <!-- Numbering inside tables is offset by one because of
         the addition of the Index property into the table. -->
    <xsl:variable name="myoidlevelnumberis">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:choose>
            <xsl:when test="$scalaroffsetfortableis">
              <xsl:value-of select="position()+1"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="position()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="name(..)='Table'">
          <xsl:value-of select="position()+1"/>
        </xsl:when>
        <xsl:otherwise>ErrorUnknownContext</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>

<!--  D a t a   S y n t a x  -->
<!-- What is the matching data syntax in SNMP? -->
    <xsl:variable name="mysyntaxis">
      <xsl:choose>
        <!-- E N U M -->
        <xsl:when test="Datatype/@Type='EnumeratedInteger'"><xsl:value-of select="$myNameis" />TC  -- Enum</xsl:when>
        <!-- I N T E G E R , with range -->
        <xsl:when test="Datatype/@Type='Integer'"><xsl:text>Integer32 </xsl:text>
            <xsl:if test="Datatype/RangeMin|Datatype/RangeMax">
                <xsl:text>(</xsl:text>
                <xsl:value-of select="Datatype/RangeMin"/>..<xsl:value-of select="Datatype/RangeMax"/>
                <xsl:text>)</xsl:text><!--  
         --></xsl:if><!--
        --><xsl:text>  -- Integer</xsl:text></xsl:when>
        <!-- G A U G E , with range -->
        <xsl:when test="Datatype/@Type='Gauge'"><xsl:text>Gauge32 </xsl:text>
            <xsl:if test="Datatype/RangeMin|Datatype/RangeMax">
                <xsl:text>(</xsl:text>
                <xsl:value-of select="Datatype/RangeMin"/>..<xsl:value-of select="Datatype/RangeMax"/>
                <xsl:text>)</xsl:text><xsl:text>  -- Gauge</xsl:text>
            </xsl:if>
        </xsl:when>
        <!-- C O U N T E R -->
        <xsl:when test="Datatype/@Type='Counter'">Counter32  -- Counter</xsl:when>
        <!-- I N T E G E R C O N S T A N T -->
        <xsl:when test="Datatype/@Type='IntegerConstant'"><xsl:text>Integer32</xsl:text>
        <xsl:text>  -- IntegerConstant</xsl:text></xsl:when>
        <!-- S T R I N G , with length -->
        <xsl:when test="Datatype/@Type='String'"><xsl:text>SnmpAdminString </xsl:text>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>(SIZE(</xsl:text></xsl:if>
            <xsl:value-of select="Datatype/LengthMin"/>..<xsl:value-of select="Datatype/LengthMax"/>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>))</xsl:text></xsl:if>
            <xsl:text>  -- String</xsl:text>
        </xsl:when>
        <!-- S T R I N G C O N S T A N T , with length -->
        <xsl:when test="Datatype/@Type='StringConstant'"><xsl:text>PdmLocalizedStringTC  -- StringConstant</xsl:text>
        </xsl:when>
        <!-- B O O L E A N -->
        <xsl:when test="Datatype/@Type='Boolean'"><xsl:text>TruthValue  -- Boolean, import from SMIv2-TC</xsl:text>
        </xsl:when>
        <!-- U R L -->
        <xsl:when test="Datatype/@Type='URL'"><xsl:text>SnmpAdminString </xsl:text>
            <xsl:text>(SIZE( 0 .. 255 ))</xsl:text>
            <xsl:text>  -- URL: USASCII, extensible to UTF-8 in future</xsl:text>
        </xsl:when>
        <!-- A B S O L U T E D A T E T I M E -->
        <xsl:when test="Datatype/@Type='AbsoluteDateTime'"><xsl:text>PdmDateTimeTC  --  absolute DateTime</xsl:text>
            <xsl:text>(SIZE( 19..25 ))</xsl:text>
            <xsl:text>  -- AbsoluteDateTime in XML format</xsl:text>
        </xsl:when>
        <!-- T I M E T I C K S -->
        <xsl:when test="Datatype/@Type='TimeTicks'"><xsl:text>TimeTicks  -- SNMP TimeTicks</xsl:text>
        </xsl:when>
        <!-- what else do we use? -->
        <!-- else unknown type -->
        <xsl:otherwise><xsl:value-of select="Datatype/@Type"/> -- ErrorUnknownDatatype</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    <xsl:text>
</xsl:text>

<!--  D e s c r i p t i o n  -->
<!-- What is the description for this property? -->
<!-- Clean up white space and remove double quotes because the
     SNMP DESCRIPTION clause is quoted. -->
    <xsl:variable name="mydescriptionis2">
      <xsl:value-of select="normalize-space(Definition)"/>
    </xsl:variable>
    <xsl:variable name="mydescriptionis">
      <xsl:value-of select="translate($mydescriptionis2,'&quot;',&quot;&apos;&quot;)"/>
    </xsl:variable>

<!--  D e f a u l t V a l u e  -->
<!-- What is the default value spec for this property? -->
<!-- Depends a lot on the datatype. -->
    <xsl:variable name="mydefvalis3">
    <xsl:if test="contains(DefaultValue/Present,'N')">"NODEFAULTVALUE"</xsl:if>
    <xsl:if test="contains(DefaultValue/Present,'Y')">
      <xsl:choose>
        <!-- enum -->
        <xsl:when test="Datatype/@Type='EnumeratedInteger'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- integer, maybe with range -->
        <xsl:when test="Datatype/@Type='Integer'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- gauge, with range -->
        <xsl:when test="Datatype/@Type='Gauge'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- counter -->
        <xsl:when test="Datatype/@Type='Counter'"><!--
        <!xx Counters not allowed to have DEFVAL xx>
        --></xsl:when>
        <!-- integer constant -->
        <xsl:when test="Datatype/@Type='IntegerConstant'"><xsl:value-of select="normalize-space(DefaultValue/Value)"/><!--
        --></xsl:when>
        <!-- string, with length -->
        <xsl:when test="Datatype/@Type='String'"><xsl:text>"</xsl:text><xsl:value-of select="translate(normalize-space(DefaultValue/Value),'&quot;',&quot;&quot;)"/><xsl:text>"</xsl:text><!--
        --></xsl:when>
        <!-- string constant, with length -->
        <xsl:when test="Datatype/@Type='StringConstant'"><xsl:text>" </xsl:text><!--<xsl:value-of select="translate(normalize-space(DefaultValue/Value),'&quot;',&quot;&quot;)"/>--><xsl:text>"</xsl:text><!--
        --></xsl:when>
        <!-- boolean -->
        <xsl:when test="Datatype/@Type='Boolean'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- URL -->
        <xsl:when test="Datatype/@Type='URL'"><xsl:text>"</xsl:text><xsl:value-of select="normalize-space(DefaultValue/Value)"/><xsl:text>"</xsl:text><!--
        --></xsl:when>
        <!-- AbsoluteDateTime -->
        <xsl:when test="Datatype/@Type='AbsoluteDateTime'"><xsl:text>"</xsl:text><xsl:value-of select="normalize-space(DefaultValue/Value)"/><xsl:text>"</xsl:text><!--
        --></xsl:when>
        <!-- TimeTicks -->
        <xsl:when test="Datatype/@Type='TimeTicks'"><xsl:text>"</xsl:text><xsl:value-of select="normalize-space(DefaultValue/Value)"/><xsl:text>"</xsl:text><!--
        --></xsl:when>
        <!-- what else do we use? -->
        <!-- add other datatype translations in here -->
        <!-- unrecognized type (yet) -->
        <xsl:otherwise>ErrorUnknownDatatype</xsl:otherwise>
      </xsl:choose> 
    </xsl:if>
    </xsl:variable>
    <!-- Compress blank or quote-quote to nothing. 
         Then put the quotes back on if it's a string type. -->
    <xsl:variable name="mydefvalis2">
      <xsl:choose>
        <!-- string -->
        <xsl:when test="Datatype/@Type='String'">"<xsl:value-of select="normalize-space(translate($mydefvalis3,'&quot;&quot;',&quot;&quot;))"/>"<!--
        --></xsl:when>
        <!-- string constant -->
        <xsl:when test="Datatype/@Type='StringConstant'">"<xsl:value-of select="normalize-space(translate($mydefvalis3,'&quot;&quot;',&quot;&quot;))"/>"<!--
        --></xsl:when>
        <xsl:otherwise><xsl:value-of select="normalize-space(translate($mydefvalis3,'&quot;&quot;',&quot;&quot;))"/><!--
        --></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- If the user specified that there is no default value, 
         then remove the funny string we stuffed in its place. -->
    <xsl:variable name="mydefvalis1">
      <xsl:choose>
        <!-- Strings with quotes -->
        <xsl:when test="$mydefvalis2 = '&quot;NODEFAULTVALUE&quot;'">""<!--
        --></xsl:when>
        <!-- Other types with no quotes -->
        <xsl:when test="$mydefvalis2 = 'NODEFAULTVALUE'">''H<!--
        --></xsl:when>
        <!-- all others -->
        <xsl:otherwise><xsl:value-of select="$mydefvalis2"/><!--
        --></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!--  T h e  D e f a u l t   D e f a u l t V a l u e  -->
    <!-- But if there is no default value specified at all, then 
            try to fake an appropriate value for the datatype. -->
    <xsl:variable name="mydefvalis">
    <xsl:if test="$mydefvalis1">
      <xsl:value-of select="normalize-space($mydefvalis1)"/>
    </xsl:if>
    <xsl:if test="not($mydefvalis1)">
      <xsl:choose>
        <!-- integer -->
        <xsl:when test="Datatype/@Type='Integer'">0<!--
        --></xsl:when>
        <!-- integer constant -->
        <xsl:when test="Datatype/@Type='IntegerConstant'">0<!--
        --></xsl:when>
        <!-- gauge -->
        <xsl:when test="Datatype/@Type='Gauge'">0<!--
        --></xsl:when>
        <!-- string -->
        <xsl:when test="Datatype/@Type='String'">""<!--
        --></xsl:when>
        <!-- string constant -->
        <xsl:when test="Datatype/@Type='StringConstant'">""<!--
        --></xsl:when>
      </xsl:choose>
    </xsl:if>
    </xsl:variable>

    <!-- U n i t -->
    <xsl:variable name="myunitis">
      <xsl:value-of select="normalize-space(Datatype/Unit)"/>
    </xsl:variable>
    <xsl:variable name="myunitphraseis">
      <xsl:if test="$myunitis"><xsl:text>
    UNITS "</xsl:text>
        <xsl:value-of select="$myunitis"/>
        <xsl:text>"</xsl:text>
      </xsl:if>
    </xsl:variable>

<!--  G e n e r a t e   O B J E C T - T Y P E  S t a t e m e n t  -->
<!-- Finally, generate the OBJECT-TYPE definition for the Property. -->
<xsl:value-of select="$mynameis"/> OBJECT-TYPE
    SYNTAX <xsl:value-of select="$mysyntaxis" />
    <xsl:choose>
    <xsl:when test="$myunitphraseis"><xsl:value-of select="$myunitphraseis"/>
    </xsl:when>
    <xsl:when test="not($myunitphraseis)"><xsl:text></xsl:text></xsl:when>
    </xsl:choose>
    MAX-ACCESS <xsl:choose>
      <xsl:when test="contains(UserWritable,'Yes')">read-write</xsl:when>
      <xsl:otherwise>read-only</xsl:otherwise>
      </xsl:choose>
    STATUS current
    DESCRIPTION 
      <xsl:text>"</xsl:text><xsl:value-of select="$mydescriptionis"/><xsl:text>"</xsl:text>
    <xsl:if test="not(Datatype/@Type='Counter')">
    DEFVAL { <xsl:value-of select='normalize-space($mydefvalis)'/> }</xsl:if>
    ::= { <xsl:value-of select="$mycontextis"/><xsl:text> </xsl:text>
        <xsl:value-of select="$myoidlevelnumberis"/> }
  </xsl:template><!--end PROPERTY-->

</xsl:stylesheet>

<!--
Edit history:
20070212    RBL v07 Original version.
20070310    RBL v10 Usable version.
20070909    RBL v12 Changed naming and numbering of scalar and table
                    properties per comments from Ira McDonald on 
                    proper SNMP naming.  
                    Add datatypes; still not complete.
20070910    RBL v13 Fix formatting for improved legibility.  
20070913    RBL v14 Add Index property at the beginning of table.
20070927    RBL v17 Put quotes around string default values.
20071009    RBL v18 Add comments where work is needed.
                    Remove some extraneous generated blank lines.  
20071013    RBL v19 Add URL and datetime datatypes.
                    Correct scalar offset in groups with tables and scalars.
20071027    RBL v20 Fix indent levels of some descriptions.
                    Put braces around DEFVALs.
                    Fix DEFVAL empty string values.
20071028    RBL v21 Add DateTime and TimeTicks datatypes.
                    Add UNITS.
20071031    RBL v22 Spell DateTime correctly as AbsoluteDateTime.
20071115    RBL v24 Reorder clauses.
                    Be more careful about DefaultValues with quotes.
20071120    RBL v25 Replace missing "::=" in group declaration.  
                    Be even more careful about DefaultValues with quotes.
                    Ensure that strings have quotes around them.
                    Force manufacturer-supplied strings (StringConstant) 
                    to have default value of null string "".  
                    Translate enum TC names to upper case prefix.  
                    Make TC names begin with capital letter.
                    (Note: do this the hard way because of a bug
                    in the Xalan translate() function, where a partial
                    match at the end of the input string succeeds and 
                    causes a substitution.) 
                    
-->

